@@ -109,21 +109,22 @@ class DistUpgradeQuirks(object):
     def PreCacheOpen(self):
         """ run before the apt cache is opened the first time """
         logging.debug("running Quirks.PreCacheOpen")
+        self._add_apport_ignore_tracker_extract()
 
     # individual quirks handler that run *after* the cache is opened
-    def jammyPostInitialUpdate(self):
+    def lunarPostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.jammyPostInitialUpdate")
+        logging.debug("running Quirks.lunarPostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
         self._test_and_fail_on_aufs()
         self._test_and_fail_on_power8()
-        self._test_and_fail_on_pam_tally()
 
         cache = self.controller.cache
         self._test_and_warn_if_ros_installed(cache)
-        self._fix_missing_nvidia_metapackage()
+
+        self._maybe_prevent_flatpak_auto_removal()
 
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -138,8 +139,8 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def jammyPostUpgrade(self):
-        logging.debug("running Quirks.jammyPostUpgrade")
+    def lunarPostUpgrade(self):
+        logging.debug("running Quirks.lunarPostUpgrade")
         cache = self.controller.cache
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -153,12 +154,12 @@ class DistUpgradeQuirks(object):
         if 'linux-firmware-raspi2' in cache:
             if cache['linux-firmware-raspi2'].is_installed:
                 self._remove_uboot_on_rpi()
-                self._remove_netplan_match_rpi()
 
     # individual quirks handler when the dpkg run is finished ---------
     def PostCleanup(self):
         " run after cleanup "
         logging.debug("running Quirks.PostCleanup")
+        self._remove_apport_ignore_tracker_extract()
 
     # run right before the first packages get installed
     def StartUpgrade(self):
@@ -374,44 +375,6 @@ class DistUpgradeQuirks(object):
                 self._view.error(summary, msg)
                 self.controller.abort()
 
-    def _test_and_fail_on_pam_tally(self):
-        """
-        Test and fail if pam_tally or pam_tally2 appears in the system's
-        PAM config for any service.  These modules have been dropped as
-        obsolete, so references to them in the config will break services
-        on upgrade.  The libpam-modules package has a preinst that will also
-        enforce this, but we want to enforce at the start of the upgrade
-        before we get halfway through an apt transaction.  LP: #1977493
-        """
-
-        for f in os.listdir('/etc/pam.d'):
-            if f in ('common-account', 'common-auth', 'common-password',
-                     'common-session', 'common-session-noninteractive'):
-                # managed by pam-auth-updates, and any references to
-                # pam_tally* will be handled by libpam-modules on upgrade
-                # without breaking
-                continue
-            with open(os.path.join('/etc/pam.d', f)) as f:
-                content = f.read()
-                if re.search('^[^#]*pam_tally', content, re.MULTILINE):
-                    logging.error("pam_tally* in use")
-                    # from libpam-modules.templates
-                    summary = _("You are using pam_tally or pam_tally2 in "
-                                "your configuration.")
-                    msg = _("The pam_tally and pam_tally2 modules have been "
-                            "removed from PAM. You are using one of these "
-                            "modules in your PAM configuration in /etc/pam.d. "
-                            "You must remove the uses of these modules before "
-                            "PAM can be upgraded; including these modules in "
-                            "your PAM configuration after the upgrade will "
-                            "stop users from being able to log into the "
-                            "system.") + "\n\n" + \
-                        _("Consider the pam_faillock module as a "
-                          "replacement for pam_tally.")
-                    self._view.error(summary, msg)
-                    self.controller.abort()
-                    return
-
     def _test_and_fail_on_power8(self):
         """
         Test and fail if running on a POWER8 processor. Starting with Ubuntu
@@ -510,6 +473,74 @@ class DistUpgradeQuirks(object):
             if not res:
                 self.controller.abort()
 
+    def _maybe_prevent_flatpak_auto_removal(self):
+        """
+        If flatpak is installed, and there are either active remotes, or
+        flatpak apps installed, prevent flatpak's auto-removal on upgrade.
+        """
+        prevent_auto_removal = False
+
+        if "flatpak" not in self.controller.cache:
+            return
+
+        if not self.controller.cache["flatpak"].is_installed:
+            return
+
+        if not os.path.exists("/usr/bin/flatpak"):
+            return
+
+        for subcmd in ["remotes", "list"]:
+            r = subprocess.run(
+                    ["/usr/bin/flatpak", subcmd],
+                    stdout=subprocess.PIPE
+            )
+            if r.stdout.decode("utf-8").strip():
+                prevent_auto_removal = True
+                break
+
+        logging.debug("flatpak will{}be marked as manually installed"
+                      .format(" " if prevent_auto_removal else " NOT "))
+
+        if not prevent_auto_removal:
+            return
+
+        self.controller.cache["flatpak"].mark_auto(auto=False)
+
+        for pkg in ("plasma-discover-backend-flatpak",
+                    "gnome-software-plugin-flatpak"):
+            if pkg not in self.controller.cache:
+                continue
+
+            if not self.controller.cache[pkg].is_installed:
+                continue
+
+            logging.debug("{} will be marked as manually installed"
+                          .format(pkg))
+            self.controller.cache[pkg].mark_auto(auto=False)
+
+        self.controller.cache.commit(
+            self._view.getAcquireProgress(),
+            self._view.getInstallProgress(self.controller.cache)
+        )
+
+    def _add_apport_ignore_tracker_extract(self):
+        path = '/etc/apport/blacklist.d/upgrade-quirk-tracker-extract-3'
+
+        try:
+            os.makedirs(os.path.dirname(path), exist_ok=True)
+            with open(path, 'w') as f:
+                f.write('/usr/libexec/tracker-extract-3\n')
+        except Exception as e:
+            logging.debug('Failed to create {}: {}'.format(path, e))
+
+    def _remove_apport_ignore_tracker_extract(self):
+        path = '/etc/apport/blacklist.d/upgrade-quirk-tracker-extract-3'
+
+        try:
+            os.remove(path)
+        except Exception as e:
+            logging.debug('Failed to remove {}: {}'.format(path, e))
+
     def _checkArmCPU(self):
         """
         parse /proc/cpuinfo and search for ARMv6 or greater
@@ -1418,116 +1449,3 @@ class DistUpgradeQuirks(object):
         except IOError as exc:
             logging.error("unable to write new boot config to %s: %s; %s",
                           boot_config_filename, exc, failure_action)
-
-    def _remove_netplan_match_rpi(self, netplan_dir='/etc/netplan'):
-        failure_action = (
-            "You may need to manually edit your netplan config "
-            "and remove the line \"driver: bcmgenet smsc95xx lan78xx\". "
-            "See LP: #1970761 for further details")
-
-        try:
-            import yaml
-        except ImportError:
-            logging.error("failed to import yaml library. %s", failure_action)
-            return
-
-        for netplan_config_file in glob.glob("%s/*.yaml" % netplan_dir):
-            try:
-                netplan_config_filename = os.path.join(netplan_dir,
-                                                       netplan_config_file)
-                with open(netplan_config_filename, 'r', encoding='utf-8') as f:
-                    orig_content = f.read()
-                    netplan_yaml = yaml.load(orig_content,
-                                             Loader=yaml.SafeLoader)
-
-            except FileNotFoundError:
-                logging.error("failed to open netplan configuration in %s; %s",
-                              netplan_config_filename, failure_action)
-                return
-
-            file_modified = False
-            try:
-                for eth in netplan_yaml["network"]["ethernets"].values():
-                    if eth["match"]["driver"] == "bcmgenet smsc95xx lan78xx":
-                        file_modified = True
-                        del eth["match"]
-                        if "set-name" in eth:
-                            del eth["set-name"]
-            except KeyError:
-                continue
-
-            if not file_modified:
-                continue
-
-            try:
-                netplan_backup_filename = (netplan_config_filename +
-                                           '.distUpgrade')
-                with open(netplan_backup_filename, 'w', encoding='utf-8') as f:
-                    f.write(orig_content)
-            except IOError as exc:
-                logging.error("unable to write netplan config backup to "
-                              "%s: %s; %s",
-                              netplan_backup_filename, exc, failure_action)
-                return
-            try:
-                with open(netplan_config_filename, 'w', encoding='utf-8') as f:
-                    yaml.dump(netplan_yaml, f, default_flow_style=False)
-            except IOError as exc:
-                logging.error("unable to write new netplan config to "
-                              "%s: %s; %s",
-                              netplan_config_filename, exc, failure_action)
-                return
-
-    def _fix_missing_nvidia_metapackage(self):
-        # LP: #1955047
-        # Desktop installs for 20.04.3 on systems with nvidia graphics don't
-        # have the driver metapackage installed
-        logging.debug(
-            "running Quirks._test_and_fix_missing_nvidia_metapackage")
-        try:
-            from NvidiaDetector.nvidiadetector import NvidiaDetection
-        except (ImportError) as e:
-            logging.error("NvidiaDetection can not be imported %s" % e)
-            return
-        nv = NvidiaDetection()
-        versions = nv.drivers.keys()
-        cache = self.controller.cache
-        # Early exit if there's a driver metapackage installed
-        for version in versions:
-            driver = 'nvidia-driver-{}'.format(version)
-            if driver in cache and cache[driver].is_installed:
-                logging.debug("package {} detected, exit.".format(driver))
-                return
-        version_to_fix = None
-        # If we find an autoremovable libnvidia-gl package, this probably
-        # means the user hit LP: #1955047
-        for version in versions:
-            nvidia_gl = 'libnvidia-gl-{}'.format(version)
-            if nvidia_gl in cache and cache[nvidia_gl].is_installed \
-               and cache[nvidia_gl].is_auto_removable:
-                version_to_fix = version
-                logging.debug("Found autoremovable package {}, assuming "
-                              "we're missing nvidia-driver-{}"
-                              .format(nvidia_gl, version))
-                break
-        if version_to_fix is None:
-            logging.debug("No autoremovable libnvidia-gl found, exiting.")
-            return
-        missing_metapackage = "nvidia-driver-{}".format(version_to_fix)
-        if missing_metapackage not in cache:
-            logging.error("Package {} not available in the APT cache!"
-                          .format(missing_metapackage))
-            summary = _("NVIDIA driver metapackage unavailable")
-            msg = _("Your system has the package libnvidia-gl-%s installed "
-                    "but the corresponding metapackage nvidia-driver-%s is "
-                    "not installed and is not available to be installed. "
-                    "The upgrade process was not able to automatically fix "
-                    "this issue and consequently the upgrade will now exit." %
-                    (version_to_fix, version_to_fix))
-            self._view.error(summary, msg)
-            self.controller.abort()
-            return
-        logging.debug("Installing missing nvidia driver metapackage {}"
-                      .format(missing_metapackage))
-        cache[missing_metapackage].mark_install()
-        cache.commit(None, None)
