@@ -2,31 +2,27 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
-import six
+import typing
 
 from cryptography import utils
 from cryptography.exceptions import (
     AlreadyFinalized,
     InvalidKey,
-    UnsupportedAlgorithm,
-    _Reasons,
 )
-from cryptography.hazmat.backends.interfaces import HMACBackend
-from cryptography.hazmat.primitives import constant_time, hmac
+from cryptography.hazmat.primitives import constant_time, hashes, hmac
 from cryptography.hazmat.primitives.kdf import KeyDerivationFunction
 
 
-@utils.register_interface(KeyDerivationFunction)
-class HKDF(object):
-    def __init__(self, algorithm, length, salt, info, backend):
-        if not isinstance(backend, HMACBackend):
-            raise UnsupportedAlgorithm(
-                "Backend object does not implement HMACBackend.",
-                _Reasons.BACKEND_MISSING_INTERFACE,
-            )
-
+class HKDF(KeyDerivationFunction):
+    def __init__(
+        self,
+        algorithm: hashes.HashAlgorithm,
+        length: int,
+        salt: typing.Optional[bytes],
+        info: typing.Optional[bytes],
+        backend: typing.Any = None,
+    ):
         self._algorithm = algorithm
 
         if salt is None:
@@ -36,42 +32,37 @@ class HKDF(object):
 
         self._salt = salt
 
-        self._backend = backend
+        self._hkdf_expand = HKDFExpand(self._algorithm, length, info)
 
-        self._hkdf_expand = HKDFExpand(self._algorithm, length, info, backend)
-
-    def _extract(self, key_material):
-        h = hmac.HMAC(self._salt, self._algorithm, backend=self._backend)
+    def _extract(self, key_material: bytes) -> bytes:
+        h = hmac.HMAC(self._salt, self._algorithm)
         h.update(key_material)
         return h.finalize()
 
-    def derive(self, key_material):
+    def derive(self, key_material: bytes) -> bytes:
         utils._check_byteslike("key_material", key_material)
         return self._hkdf_expand.derive(self._extract(key_material))
 
-    def verify(self, key_material, expected_key):
+    def verify(self, key_material: bytes, expected_key: bytes) -> None:
         if not constant_time.bytes_eq(self.derive(key_material), expected_key):
             raise InvalidKey
 
 
-@utils.register_interface(KeyDerivationFunction)
-class HKDFExpand(object):
-    def __init__(self, algorithm, length, info, backend):
-        if not isinstance(backend, HMACBackend):
-            raise UnsupportedAlgorithm(
-                "Backend object does not implement HMACBackend.",
-                _Reasons.BACKEND_MISSING_INTERFACE,
-            )
-
+class HKDFExpand(KeyDerivationFunction):
+    def __init__(
+        self,
+        algorithm: hashes.HashAlgorithm,
+        length: int,
+        info: typing.Optional[bytes],
+        backend: typing.Any = None,
+    ):
         self._algorithm = algorithm
 
-        self._backend = backend
-
         max_length = 255 * algorithm.digest_size
 
         if length > max_length:
             raise ValueError(
-                "Can not derive keys larger than {} octets.".format(max_length)
+                "Cannot derive keys larger than {} octets.".format(max_length)
             )
 
         self._length = length
@@ -85,21 +76,21 @@ class HKDFExpand(object):
 
         self._used = False
 
-    def _expand(self, key_material):
+    def _expand(self, key_material: bytes) -> bytes:
         output = [b""]
         counter = 1
 
         while self._algorithm.digest_size * (len(output) - 1) < self._length:
-            h = hmac.HMAC(key_material, self._algorithm, backend=self._backend)
+            h = hmac.HMAC(key_material, self._algorithm)
             h.update(output[-1])
             h.update(self._info)
-            h.update(six.int2byte(counter))
+            h.update(bytes([counter]))
             output.append(h.finalize())
             counter += 1
 
         return b"".join(output)[: self._length]
 
-    def derive(self, key_material):
+    def derive(self, key_material: bytes) -> bytes:
         utils._check_byteslike("key_material", key_material)
         if self._used:
             raise AlreadyFinalized
@@ -107,6 +98,6 @@ class HKDFExpand(object):
         self._used = True
         return self._expand(key_material)
 
-    def verify(self, key_material, expected_key):
+    def verify(self, key_material: bytes, expected_key: bytes) -> None:
         if not constant_time.bytes_eq(self.derive(key_material), expected_key):
             raise InvalidKey
