@@ -8,18 +8,15 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-# pylint: disable=too-many-branches
-
 import fnmatch
 import inspect
 import logging
 import os
 import re
 
-from shlex import quote
+from pipes import quote
 from sos.policies import load
 from sos.policies.init_systems import InitSystem
-from sos.collector.transports.juju import JujuSSH
 from sos.collector.transports.control_persist import SSHControlPersist
 from sos.collector.transports.local import LocalTransport
 from sos.collector.transports.oc import OCTransport
@@ -28,14 +25,13 @@ from sos.collector.exceptions import (CommandTimeoutException,
                                       ConnectionException,
                                       UnsupportedHostException,
                                       InvalidTransportException)
-from sos.utilities import sos_parse_version
+from sos.utilities import parse_version
 
 TRANSPORTS = {
     'local': LocalTransport,
     'control_persist': SSHControlPersist,
     'oc': OCTransport,
-    'saltstack': SaltStackMaster,
-    'juju': JujuSSH,
+    'saltstack': SaltStackMaster
 }
 
 
@@ -79,14 +75,10 @@ class SosNode():
         self.soslog = logging.getLogger('sos')
         self.ui_log = logging.getLogger('sos_ui')
         self._transport = self._load_remote_transport(commons)
-        # Overwrite need_sudo if transports default_user
-        # is set and is not root.
-        if self._transport.default_user:
-            self.need_sudo = self._transport.default_user != 'root'
         try:
             self._transport.connect(self._password)
         except Exception as err:
-            self.log_error(f'Unable to open remote session: {err}')
+            self.log_error('Unable to open remote session: %s' % err)
             raise
         # load the host policy now, even if we don't want to load further
         # host information. This is necessary if we're running locally on the
@@ -130,14 +122,14 @@ class SosNode():
             return TRANSPORTS[self.opts.transport](self.address, commons)
         elif self.opts.transport != 'auto':
             self.log_error(
-                "Connection failed: unknown or unsupported transport "
-                f"{self.opts.transport}"
+                "Connection failed: unknown or unsupported transport %s"
+                % self.opts.transport
             )
             raise InvalidTransportException(self.opts.transport)
         return SSHControlPersist(self.address, commons)
 
     def _fmt_msg(self, msg):
-        return f"{self._hostname:<{self.hostlen + 1}} : {msg}"
+        return '{:<{}} : {}'.format(self._hostname, self.hostlen + 1, msg)
 
     @property
     def env_vars(self):
@@ -204,18 +196,17 @@ class SosNode():
                 ret = self.run_command(self.host.restart_sos_container(),
                                        need_root=True)
                 if ret['status'] == 0:
-                    self.log_info("Temporary container "
-                                  f"{self.host.sos_container_name} created")
+                    self.log_info("Temporary container %s created"
+                                  % self.host.sos_container_name)
                     return True
                 else:
-                    self.log_error("Could not start container after create: "
-                                   f"{ret['output']}")
+                    self.log_error("Could not start container after create: %s"
+                                   % ret['output'])
                     raise Exception
             else:
-                self.log_error("Could not create container on host: "
-                               f"{res['output']}")
+                self.log_error("Could not create container on host: %s"
+                               % res['output'])
                 raise Exception
-        return False
 
     def get_container_auth(self):
         """Determine what the auth string should be to pull the image used to
@@ -234,7 +225,7 @@ class SosNode():
     def file_exists(self, fname, need_root=False):
         """Checks for the presence of fname on the remote node"""
         try:
-            res = self.run_command(f"stat {fname}", need_root=need_root)
+            res = self.run_command("stat %s" % fname, need_root=need_root)
             return res['status'] == 0
         except Exception:
             return False
@@ -258,20 +249,20 @@ class SosNode():
     def log_info(self, msg):
         """Used to print and log info messages"""
         caller = inspect.stack()[1][3]
-        lmsg = f'[{self._hostname}:{caller}] {msg}'
+        lmsg = '[%s:%s] %s' % (self._hostname, caller, msg)
         self.soslog.info(lmsg)
 
     def log_error(self, msg):
         """Used to print and log error messages"""
         caller = inspect.stack()[1][3]
-        lmsg = f'[{self._hostname}:{caller}] {msg}'
+        lmsg = '[%s:%s] %s' % (self._hostname, caller, msg)
         self.soslog.error(lmsg)
 
     def log_debug(self, msg):
         """Used to print and log debug messages"""
         msg = self._sanitize_log_msg(msg)
         caller = inspect.stack()[1][3]
-        msg = f'[{self._hostname}:{caller}] {msg}'
+        msg = '[%s:%s] %s' % (self._hostname, caller, msg)
         self.soslog.debug(msg)
 
     def _format_cmd(self, cmd):
@@ -279,9 +270,9 @@ class SosNode():
         here we prefix the command with the correct bits
         """
         if self.opts.become_root:
-            return f"su -c {quote(cmd)}"
+            return "su -c %s" % quote(cmd)
         if self.need_sudo:
-            return f"sudo -S {cmd}"
+            return "sudo -S %s" % cmd
         return cmd
 
     def _load_sos_info(self):
@@ -309,14 +300,14 @@ class SosNode():
                 # comparison by parse_version
                 ver += '.0'
             try:
-                ver += f'-{rel.split(".")[0]}'
+                ver += '-%s' % rel.split('.')[0]
             except Exception as err:
-                self.log_debug(f"Unable to fully parse sos release: {err}")
+                self.log_debug("Unable to fully parse sos release: %s" % err)
 
         self.sos_info['version'] = ver
 
         if self.sos_info['version']:
-            self.log_info(f'sos version is {self.sos_info["version"]}')
+            self.log_info('sos version is %s' % self.sos_info['version'])
         else:
             if not self.address == self.opts.primary:
                 # in the case where the 'primary' enumerates nodes but is not
@@ -328,16 +319,15 @@ class SosNode():
         # sos-4.0 changes the binary
         if self.check_sos_version('4.0'):
             self.sos_bin = 'sos report'
-        cmd = f"{self.sos_bin} -l"
+        cmd = "%s -l" % self.sos_bin
         sosinfo = self.run_command(cmd, use_container=True, need_root=True)
         if sosinfo['status'] == 0:
             self._load_sos_plugins(sosinfo['output'])
         if self.check_sos_version('3.6'):
             self._load_sos_presets()
-        return None
 
     def _load_sos_presets(self):
-        cmd = f'{self.sos_bin} --list-presets'
+        cmd = '%s --list-presets' % self.sos_bin
         res = self.run_command(cmd, use_container=True, need_root=True)
         if res['status'] == 0:
             for line in res['output'].splitlines():
@@ -369,8 +359,8 @@ class SosNode():
                 if not is_list:
                     try:
                         res.append(line.split()[0])
-                    except Exception as err:
-                        self.log_debug(f"Error parsing sos help: {err}")
+                    except Exception:
+                        pass
                 else:
                     r = line.split(',')
                     res.extend(p.strip() for p in r if p.strip())
@@ -379,10 +369,10 @@ class SosNode():
     def read_file(self, to_read):
         """Reads the specified file and returns the contents"""
         try:
-            self.log_info(f"Reading file {to_read}")
+            self.log_info("Reading file %s" % to_read)
             return self._transport.read_file(to_read)
         except Exception as err:
-            self.log_error(f"Exception while reading {to_read}: {err}")
+            self.log_error("Exception while reading %s: %s" % (to_read, err))
             return ''
 
     def determine_host_policy(self):
@@ -390,15 +380,15 @@ class SosNode():
         distributions
         """
         if self.local:
-            self.log_info(
-                f"using local policy {self.commons['policy'].distro}")
+            self.log_info("using local policy %s"
+                          % self.commons['policy'].distro)
             return self.commons['policy']
         host = load(cache={}, sysroot=self.opts.sysroot, init=InitSystem(),
                     probe_runtime=True,
                     remote_exec=self._transport.run_command,
                     remote_check=self.read_file('/etc/os-release'))
         if host:
-            self.log_info(f"loaded policy {host.distro} for host")
+            self.log_info("loaded policy %s for host" % host.distro)
             return host
         self.log_error('Unable to determine host installation. Ignoring node')
         raise UnsupportedHostException
@@ -414,11 +404,29 @@ class SosNode():
         :returns:   True if installed version is at least ``ver``, else False
         :rtype:     ``bool``
         """
+        def _format_version(ver):
+            # format the version we're checking to a standard form of X.Y.Z-R
+            try:
+                _fver = ver.split('-')[0]
+                _rel = ''
+                if '-' in ver:
+                    _rel = '-' + ver.split('-')[-1].split('.')[0]
+                if len(_fver.split('.')) == 2:
+                    _fver += '.0'
+
+                return _fver + _rel
+            except Exception as err:
+                self.log_debug("Unable to format '%s': %s" % (ver, err))
+                return ver
+
+        _ver = _format_version(ver)
+
         try:
-            _node_ver = self.sos_info['version']
-            return sos_parse_version(_node_ver) >= sos_parse_version(ver)
+            _node_ver = parse_version(self.sos_info['version'])
+            _test_ver = parse_version(_ver)
+            return _node_ver >= _test_ver
         except Exception as err:
-            self.log_error(f"Error checking sos version: {err}")
+            self.log_error("Error checking sos version: %s" % err)
             return False
 
     def is_installed(self, pkg):
@@ -427,25 +435,20 @@ class SosNode():
             return False
         return self.host.package_manager.pkg_by_name(pkg) is not None
 
-    def run_command(self, cmd, timeout=180, use_shell='auto', need_root=False,
+    def run_command(self, cmd, timeout=180, get_pty=False, need_root=False,
                     use_container=False, env=None):
         """Runs a given cmd, either via the SSH session or locally
 
-        :param cmd:     The full command to be run
-        :type cmd:      ``str``
-
-        :param timeout: Time in seconds to wait for `cmd` to complete
-        :type timeout:  ``int``
-
-        :param use_shell: If a shell is needed to run `cmd`, set to True
-        :type use_shell:  ``bool`` or ``auto`` for transport-determined
-
-        :param use_container: Run this command in a container *IF* the host
-                              is a containerized host
-        :type use_container: ``bool``
-
-        :param env: Pass environment variables to set for this `cmd`
-        :type env:  ``dict``
+        Arguments:
+            cmd - the full command to be run
+            timeout - time in seconds to wait for the command to complete
+            get_pty - If a shell is absolutely needed to run a command, set
+                      this to True
+            need_root - if a command requires root privileges, setting this to
+                        True tells sos-collector to format the command with
+                        sudo or su - as appropriate and to input the password
+            use_container - Run this command in a container *IF* the host is
+                            containerized
         """
         if not self.connected and not self.local:
             self.log_debug('Node is disconnected, attempting to reconnect')
@@ -455,17 +458,21 @@ class SosNode():
                     self.log_debug('Failed to reconnect to node')
                     raise ConnectionException
             except Exception as err:
-                self.log_debug(f"Error while trying to reconnect: {err}")
+                self.log_debug("Error while trying to reconnect: %s" % err)
                 raise
         if use_container and self.host.containerized:
             cmd = self.host.format_container_command(cmd)
         if need_root:
             cmd = self._format_cmd(cmd)
+
+        if 'atomic' in cmd:
+            get_pty = True
+
         if env:
             _cmd_env = self.env_vars
             env = _cmd_env.update(env)
         return self._transport.run_command(cmd, timeout, need_root, env,
-                                           use_shell)
+                                           get_pty)
 
     def sosreport(self):
         """Run an sos report on the node, then collect it"""
@@ -477,8 +484,8 @@ class SosNode():
                 self.log_error('Unable to determine path of sos archive')
             if self.sos_path:
                 self.retrieved = self.retrieve_sosreport()
-        except Exception as err:
-            self.log_error(f"Error during sos execution: {err}")
+        except Exception:
+            pass
         self.cleanup()
 
     def _preset_exists(self, preset):
@@ -538,9 +545,9 @@ class SosNode():
             if not self.preset:
                 self.preset = self.cluster.sos_preset
             else:
-                self.log_info('Cluster specified preset '
-                              f'{self.cluster.sos_preset} but user has also '
-                              'defined a preset. Using user specification.')
+                self.log_info('Cluster specified preset %s but user has also '
+                              'defined a preset. Using user specification.'
+                              % self.cluster.sos_preset)
         if self.cluster.sos_plugins:
             for plug in self.cluster.sos_plugins:
                 if plug not in self.enable_plugins:
@@ -555,7 +562,8 @@ class SosNode():
         if self.cluster.sos_plugin_options:
             for opt in self.cluster.sos_plugin_options:
                 if not any(opt in o for o in self.plugopts):
-                    option = f'{opt}={self.cluster.sos_plugin_options[opt]}'
+                    option = '%s=%s' % (opt,
+                                        self.cluster.sos_plugin_options[opt])
                     self.plugopts.append(option)
 
         # set primary-only options
@@ -583,7 +591,7 @@ class SosNode():
         sos_cmd = self.sos_info['sos_cmd']
         label = self.determine_sos_label()
         if label:
-            sos_cmd = f'{sos_cmd} {quote(label)} '
+            sos_cmd = '%s %s ' % (sos_cmd, quote(label))
 
         sos_opts = []
 
@@ -591,13 +599,13 @@ class SosNode():
         if self.check_sos_version('3.6'):
             # 4 threads is the project's default
             if self.opts.threads != 4:
-                sos_opts.append(f'--threads={quote(str(self.opts.threads))}')
+                sos_opts.append('--threads=%s' % quote(str(self.opts.threads)))
 
         # sos-3.7 added options
         if self.check_sos_version('3.7'):
             if self.opts.plugin_timeout:
-                sos_opts.append(
-                    f'--plugin-timeout={quote(str(self.opts.plugin_timeout))}')
+                sos_opts.append('--plugin-timeout=%s'
+                                % quote(str(self.opts.plugin_timeout)))
 
         # sos-3.8 added options
         if self.check_sos_version('3.8'):
@@ -608,40 +616,35 @@ class SosNode():
                 sos_opts.append('--no-env-vars')
 
             if self.opts.since:
-                sos_opts.append(f'--since={quote(self.opts.since)}')
+                sos_opts.append('--since=%s' % quote(self.opts.since))
 
         if self.check_sos_version('4.1'):
             if self.opts.skip_commands:
-                sos_opts.append('--skip-commands='
-                                f'{quote(",".join(self.opts.skip_commands))}')
+                sos_opts.append(
+                    '--skip-commands=%s' % (
+                        quote(','.join(self.opts.skip_commands)))
+                )
             if self.opts.skip_files:
                 sos_opts.append(
-                    f'--skip-files={quote(",".join(self.opts.skip_files))}'
+                    '--skip-files=%s' % (quote(','.join(self.opts.skip_files)))
                 )
 
         if self.check_sos_version('4.2'):
             if self.opts.cmd_timeout:
-                sos_opts.append(
-                    f'--cmd-timeout={quote(str(self.opts.cmd_timeout))}'
-                )
+                sos_opts.append('--cmd-timeout=%s'
+                                % quote(str(self.opts.cmd_timeout)))
 
         # handle downstream versions that backported this option
         if self.check_sos_version('4.3') or self.check_sos_version('4.2-13'):
             if self.opts.container_runtime != 'auto':
                 sos_opts.append(
-                    f"--container-runtime={self.opts.container_runtime}"
+                    "--container-runtime=%s" % self.opts.container_runtime
                 )
             if self.opts.namespaces:
                 sos_opts.append(
-                    f"--namespaces={self.opts.namespaces}"
+                    "--namespaces=%s" % self.opts.namespaces
                 )
 
-        if self.check_sos_version('4.5.2'):
-            if self.opts.journal_size:
-                sos_opts.append(f"--journal-size={self.opts.journal_size}")
-            if self.opts.low_priority:
-                sos_opts.append('--low-priority')
-
         self.update_cmd_from_cluster()
 
         sos_cmd = sos_cmd.replace(
@@ -658,26 +661,26 @@ class SosNode():
                     if self._plugin_exists(o.split('.')[0])
                     and self._plugin_option_exists(o.split('=')[0])]
             if opts:
-                sos_opts.append(f'-k {quote(",".join(o for o in opts))}')
+                sos_opts.append('-k %s' % quote(','.join(o for o in opts)))
 
         if self.preset:
             if self._preset_exists(self.preset):
-                sos_opts.append(f'--preset={quote(self.preset)}')
+                sos_opts.append('--preset=%s' % quote(self.preset))
             else:
-                self.log_debug(f'Requested to enable preset {self.preset} but '
-                               'preset does not exist on node')
+                self.log_debug('Requested to enable preset %s but preset does '
+                               'not exist on node' % self.preset)
 
         if self.only_plugins:
             plugs = [o for o in self.only_plugins if self._plugin_exists(o)]
             if len(plugs) != len(self.only_plugins):
                 not_only = list(set(self.only_plugins) - set(plugs))
-                self.log_debug(f'Requested plugins {not_only} were requested '
-                               'to be enabled but do not exist')
+                self.log_debug('Requested plugins %s were requested to be '
+                               'enabled but do not exist' % not_only)
             only = self._fmt_sos_opt_list(self.only_plugins)
             if only:
-                sos_opts.append(f'--only-plugins={quote(only)}')
-            self.sos_cmd = f"{sos_cmd} {' '.join(sos_opts)}"
-            self.log_info(f'Final sos command set to {self.sos_cmd}')
+                sos_opts.append('--only-plugins=%s' % quote(only))
+            self.sos_cmd = "%s %s" % (sos_cmd, ' '.join(sos_opts))
+            self.log_info('Final sos command set to %s' % self.sos_cmd)
             self.manifest.add_field('final_sos_command', self.sos_cmd)
             return
 
@@ -686,11 +689,11 @@ class SosNode():
             skip = [o for o in self.skip_plugins if self._check_enabled(o)]
             if len(skip) != len(self.skip_plugins):
                 not_skip = list(set(self.skip_plugins) - set(skip))
-                self.log_debug(f'Requested to skip plugins {not_skip}, but '
-                               'plugins are already not enabled')
+                self.log_debug('Requested to skip plugins %s, but plugins are '
+                               'already not enabled' % not_skip)
             skipln = self._fmt_sos_opt_list(skip)
             if skipln:
-                sos_opts.append(f'--skip-plugins={quote(skipln)}')
+                sos_opts.append('--skip-plugins=%s' % quote(skipln))
 
         if self.enable_plugins:
             # only run enable for plugins that are disabled
@@ -699,14 +702,14 @@ class SosNode():
                     and self._check_disabled(o) and self._plugin_exists(o)]
             if len(opts) != len(self.enable_plugins):
                 not_on = list(set(self.enable_plugins) - set(opts))
-                self.log_debug(f'Requested to enable plugins {not_on}, but '
-                               'plugins are already enabled or do not exist')
+                self.log_debug('Requested to enable plugins %s, but plugins '
+                               'are already enabled or do not exist' % not_on)
             enable = self._fmt_sos_opt_list(opts)
             if enable:
-                sos_opts.append(f'--enable-plugins={quote(enable)}')
+                sos_opts.append('--enable-plugins=%s' % quote(enable))
 
-        self.sos_cmd = f"{sos_cmd} {' '.join(sos_opts)}"
-        self.log_info(f'Final sos command set to {self.sos_cmd}')
+        self.sos_cmd = "%s %s" % (sos_cmd, ' '.join(sos_opts))
+        self.log_info('Final sos command set to %s' % self.sos_cmd)
         self.manifest.add_field('final_sos_command', self.sos_cmd)
 
     def determine_sos_label(self):
@@ -715,19 +718,19 @@ class SosNode():
         label += self.cluster.get_node_label(self)
 
         if self.opts.label:
-            label += (f'{self.opts.label}' if not label
-                      else f'-{self.opts.label}')
+            label += ('%s' % self.opts.label if not label
+                      else '-%s' % self.opts.label)
 
         if not label:
             return None
 
-        self.log_debug(f'Label for sos report set to {label}')
+        self.log_debug('Label for sos report set to %s' % label)
         if self.check_sos_version('3.6'):
             lcmd = '--label'
         else:
             lcmd = '--name'
-            label = f'{self.address.split(".")[0]}-{label}'
-        return f'{lcmd}={label}'
+            label = '%s-%s' % (self.address.split('.')[0], label)
+        return '%s=%s' % (lcmd, label)
 
     def finalize_sos_path(self, path):
         """Use host facts to determine if we need to change the sos path
@@ -736,7 +739,7 @@ class SosNode():
         if pstrip:
             path = path.replace(pstrip, '')
         path = path.split()[0]
-        self.log_info(f'Final sos path: {path}')
+        self.log_info('Final sos path: %s' % path)
         self.sos_path = path
         self.archive = path.split('/')[-1]
         self.manifest.add_field('collected_archive', self.archive)
@@ -752,7 +755,7 @@ class SosNode():
         if len(stdout) > 0:
             return stdout.split('\n')[0:1]
         else:
-            return f'sos exited with code {rc}'
+            return 'sos exited with code %s' % rc
 
     def execute_sos_command(self):
         """Run sos report and capture the resulting file path"""
@@ -762,8 +765,7 @@ class SosNode():
             checksum = False
             res = self.run_command(self.sos_cmd,
                                    timeout=self.opts.timeout,
-                                   use_shell=True,
-                                   need_root=True,
+                                   get_pty=True, need_root=True,
                                    use_container=True,
                                    env=self.sos_env_vars)
             if res['status'] == 0:
@@ -787,16 +789,15 @@ class SosNode():
                     self.manifest.add_field('checksum_type', 'unknown')
             else:
                 err = self.determine_sos_error(res['status'], res['output'])
-                self.log_debug("Error running sos report. rc = "
-                               f"{res['status']} msg = {res['output']}")
+                self.log_debug("Error running sos report. rc = %s msg = %s"
+                               % (res['status'], res['output']))
                 raise Exception(err)
             return path
         except CommandTimeoutException:
             self.log_error('Timeout exceeded')
             raise
-        except Exception as err:
-            self.log_info(f"Exception during sos report execution: {err}")
-            self.ui_msg(f"Error running sos report: {err}")
+        except Exception as e:
+            self.log_error('Error running sos report: %s' % e)
             raise
 
     def retrieve_file(self, path):
@@ -805,14 +806,15 @@ class SosNode():
         dest = os.path.join(destdir, path.split('/')[-1])
         try:
             if self.file_exists(path):
-                self.log_info(f"Copying remote {path} to local {destdir}")
+                self.log_info("Copying remote %s to local %s" %
+                              (path, destdir))
                 return self._transport.retrieve_file(path, dest)
             else:
-                self.log_debug(f"Attempting to copy remote file {path}, but it"
-                               " does not exist on filesystem")
+                self.log_debug("Attempting to copy remote file %s, but it "
+                               "does not exist on filesystem" % path)
                 return False
         except Exception as err:
-            self.log_debug(f"Failed to retrieve {path}: {err}")
+            self.log_debug("Failed to retrieve %s: %s" % (path, err))
             return False
 
     def remove_file(self, path):
@@ -822,43 +824,54 @@ class SosNode():
         path = ''.join(path.split())
         try:
             if len(path.split('/')) <= 2:  # ensure we have a non '/' path
-                self.log_debug(f"Refusing to remove path {path}: appears to "
-                               "be incorrect and possibly dangerous")
+                self.log_debug("Refusing to remove path %s: appears to be "
+                               "incorrect and possibly dangerous" % path)
                 return False
             if self.file_exists(path):
-                self.log_info(f"Removing file {path}")
-                cmd = f"rm -f {path}"
+                self.log_info("Removing file %s" % path)
+                cmd = "rm -f %s" % path
                 res = self.run_command(cmd, need_root=True)
                 return res['status'] == 0
             else:
-                self.log_debug(f"Attempting to remove remote file {path}, but "
-                               "it does not exist on filesystem")
+                self.log_debug("Attempting to remove remote file %s, but it "
+                               "does not exist on filesystem" % path)
                 return False
         except Exception as e:
-            self.log_debug(f'Failed to remove {path}: {e}')
+            self.log_debug('Failed to remove %s: %s' % (path, e))
             return False
 
     def retrieve_sosreport(self):
         """Collect the sosreport archive from the node"""
-        if self.need_sudo or self.opts.become_root:
+        if self.sos_path:
+            if self.need_sudo or self.opts.become_root:
+                try:
+                    self.make_archive_readable(self.sos_path)
+                except Exception:
+                    self.log_error('Failed to make archive readable')
+                    return False
+            self.log_info('Retrieving sos report from %s' % self.address)
+            self.ui_msg('Retrieving sos report...')
             try:
-                self.make_archive_readable(self.sos_path)
-            except Exception:
-                self.log_error('Failed to make archive readable')
+                ret = self.retrieve_file(self.sos_path)
+            except Exception as err:
+                self.log_error(err)
+                return False
+            if ret:
+                self.ui_msg('Successfully collected sos report')
+                self.file_list.append(self.sos_path.split('/')[-1])
+                return True
+            else:
+                self.ui_msg('Failed to retrieve sos report')
                 return False
-        self.log_info(f'Retrieving sos report from {self.address}')
-        self.ui_msg('Retrieving sos report...')
-        try:
-            ret = self.retrieve_file(self.sos_path)
-        except Exception as err:
-            self.log_error(err)
-            return False
-        if ret:
-            self.ui_msg('Successfully collected sos report')
-            self.file_list.append(self.sos_path.split('/')[-1])
-            return True
         else:
-            self.ui_msg('Failed to retrieve sos report')
+            # sos sometimes fails but still returns a 0 exit code
+            if self.stderr.read():
+                e = self.stderr.read()
+            else:
+                e = [x.strip() for x in self.stdout.readlines() if x.strip][-1]
+            self.soslog.error(
+                'Failed to run sos report on %s: %s' % (self.address, e))
+            self.log_error('Failed to run sos report. %s' % e)
             return False
 
     def remove_sos_archive(self):
@@ -869,8 +882,8 @@ class SosNode():
             # is no archive at the original location to remove
             return
         if 'sosreport' not in self.sos_path:
-            self.log_debug(f"Node sos report path {self.sos_path} looks "
-                           "incorrect. Not attempting to remove path")
+            self.log_debug("Node sos report path %s looks incorrect. Not "
+                           "attempting to remove path" % self.sos_path)
             return
         removed = self.remove_file(self.sos_path)
         if not removed:
@@ -895,18 +908,18 @@ class SosNode():
                     try:
                         self.make_archive_readable(filename)
                     except Exception as err:
-                        self.log_error(f"Unable to retrieve file {filename}")
-                        self.log_debug(f"Failed to make file {filename} "
-                                       f"readable: {err}")
+                        self.log_error("Unable to retrieve file %s" % filename)
+                        self.log_debug("Failed to make file %s readable: %s"
+                                       % (filename, err))
                         continue
                 ret = self.retrieve_file(filename)
                 if ret:
                     self.file_list.append(filename.split('/')[-1])
                     self.remove_file(filename)
                 else:
-                    self.log_error(f"Unable to retrieve file {filename}")
+                    self.log_error("Unable to retrieve file %s" % filename)
             except Exception as e:
-                msg = f'Error collecting additional data from primary: {e}'
+                msg = 'Error collecting additional data from primary: %s' % e
                 self.log_error(msg)
 
     def make_archive_readable(self, filepath):
@@ -915,7 +928,7 @@ class SosNode():
 
         This is only used when we're not connecting as root.
         """
-        cmd = f'chmod o+r {filepath}'
+        cmd = 'chmod o+r %s' % filepath
         res = self.run_command(cmd, timeout=10, need_root=True)
         if res['status'] == 0:
             return True
