@@ -49,10 +49,7 @@ class Docker(Plugin, CosPlugin):
 
         subcmds = [
             'events --since 24h --until 1s',
-            'info',
-            'images',
             'ps',
-            'ps -a',
             'stats --no-stream',
             'version',
             'volume ls'
@@ -61,6 +58,13 @@ class Docker(Plugin, CosPlugin):
         for subcmd in subcmds:
             self.add_cmd_output("docker %s" % subcmd)
 
+        self.add_cmd_output("docker info",
+                            tags="docker_info")
+        self.add_cmd_output("docker images",
+                            tags="docker_images")
+        self.add_cmd_output("docker ps -a",
+                            tags="docker_list_containers")
+
         # separately grab these separately as they can take a *very* long time
         if self.get_option('size'):
             self.add_cmd_output('docker ps -as', priority=100)
@@ -90,7 +94,8 @@ class Docker(Plugin, CosPlugin):
         for img in images:
             name, img_id = img
             insp = name if 'none' not in name else img_id
-            self.add_cmd_output("docker inspect %s" % insp, subdir='images')
+            self.add_cmd_output("docker inspect %s" % insp, subdir='images',
+                                tags="docker_image_inspect")
 
         for vol in volumes:
             self.add_cmd_output("docker volume inspect %s" % vol,
@@ -135,7 +140,6 @@ class UbuntuDocker(Docker, UbuntuPlugin):
     def setup(self):
         super(UbuntuDocker, self).setup()
         self.add_copy_spec([
-            "/etc/containerd/",
             "/etc/default/docker",
             "/run/docker/libcontainerd/containerd/events.log"
         ])
