@@ -14,7 +14,7 @@ from typing import Dict, Iterable, List, NamedTuple, Optional, Union
 import apt  # type: ignore
 import apt_pkg  # type: ignore
 
-from uaclient import event_logger, exceptions, gpg, messages, system, util
+from uaclient import event_logger, exceptions, gpg, messages, system
 from uaclient.defaults import ESM_APT_ROOTDIR
 
 APT_HELPER_TIMEOUT = 60.0  # 60 second timeout used for apt-helper call
@@ -56,7 +56,6 @@ deb https://esm.ubuntu.com/{name}/ubuntu {series}-{name}-updates main
 APT_RETRIES = [1.0, 5.0, 10.0]
 
 event = event_logger.get_event_logger()
-LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
 
 
 @enum.unique
@@ -65,8 +64,8 @@ class AptProxyScope(enum.Enum):
     UACLIENT = object()
 
 
-InstalledAptPackage = NamedTuple(
-    "InstalledAptPackage", [("name", str), ("version", str), ("arch", str)]
+InstalledAptPackages = NamedTuple(
+    "InstalledAptPackages", [("name", str), ("version", str), ("arch", str)]
 )
 
 
@@ -89,7 +88,7 @@ def assert_valid_apt_credentials(repo_url, username, password):
                 [
                     "/usr/lib/apt/apt-helper",
                     "download-file",
-                    "{}://{}:{}@{}/pool/".format(
+                    "{}://{}:{}@{}/ubuntu/pool/".format(
                         protocol, username, password, repo_path
                     ),
                     os.path.join(tmpd, "apt-helper-output"),
@@ -170,24 +169,21 @@ def _parse_apt_update_for_invalid_apt_config(
 def run_apt_command(
     cmd: List[str],
     error_msg: Optional[str] = None,
-    override_env_vars: Optional[Dict[str, str]] = None,
+    env: Optional[Dict[str, str]] = {},
 ) -> str:
     """Run an apt command, retrying upon failure APT_RETRIES times.
 
     :param cmd: List containing the apt command to run, passed to subp.
     :param error_msg: The string to raise as UserFacingError when all retries
        are exhausted in failure.
-    :param override_env_vars: Passed directly as subp's override_env_vars arg
+    :param env: Optional dictionary of environment variables to pass to subp.
 
     :return: stdout from successful run of the apt command.
     :raise UserFacingError: on issues running apt-cache policy.
     """
     try:
         out, _err = system.subp(
-            cmd,
-            capture=True,
-            retry_sleeps=APT_RETRIES,
-            override_env_vars=override_env_vars,
+            cmd, capture=True, retry_sleeps=APT_RETRIES, env=env
         )
     except exceptions.ProcessExecutionError as e:
         if "Could not get lock /var/lib/dpkg/lock" in str(e.stderr):
@@ -212,12 +208,10 @@ def run_apt_command(
 @lru_cache(maxsize=None)
 def get_apt_cache_policy(
     error_msg: Optional[str] = None,
-    override_env_vars: Optional[Dict[str, str]] = None,
+    env: Optional[Dict[str, str]] = {},
 ) -> str:
     return run_apt_command(
-        cmd=["apt-cache", "policy"],
-        error_msg=error_msg,
-        override_env_vars=override_env_vars,
+        cmd=["apt-cache", "policy"], error_msg=error_msg, env=env
     )
 
 
@@ -270,9 +264,7 @@ def get_esm_cache():
     return cache
 
 
-def get_pkg_candidate_version(
-    pkg: str, check_esm_cache: bool = False
-) -> Optional[str]:
+def get_pkg_candidate_version(pkg: str) -> Optional[str]:
     with PreserveAptCfg(get_apt_cache) as cache:
         try:
             package = cache[pkg]
@@ -286,8 +278,6 @@ def get_pkg_candidate_version(
 
     if not pkg_candidate:
         return None
-    elif not check_esm_cache:
-        return pkg_candidate
 
     with PreserveAptCfg(get_esm_cache) as esm_cache:
         if esm_cache:
@@ -313,22 +303,16 @@ def get_pkg_candidate_version(
 def get_apt_cache_policy_for_package(
     package: str,
     error_msg: Optional[str] = None,
-    override_env_vars: Optional[Dict[str, str]] = None,
+    env: Optional[Dict[str, str]] = {},
 ) -> str:
     return run_apt_command(
-        cmd=["apt-cache", "policy", package],
-        error_msg=error_msg,
-        override_env_vars=override_env_vars,
+        cmd=["apt-cache", "policy", package], error_msg=error_msg, env=env
     )
 
 
-def run_apt_update_command(
-    override_env_vars: Optional[Dict[str, str]] = None
-) -> str:
+def run_apt_update_command(env: Optional[Dict[str, str]] = {}) -> str:
     try:
-        out = run_apt_command(
-            cmd=["apt-get", "update"], override_env_vars=override_env_vars
-        )
+        out = run_apt_command(cmd=["apt-get", "update"], env=env)
     except exceptions.APTProcessConflictError:
         raise exceptions.APTUpdateProcessConflictError()
     except exceptions.APTInvalidRepoError as e:
@@ -351,7 +335,7 @@ def run_apt_install_command(
     packages: List[str],
     apt_options: Optional[List[str]] = None,
     error_msg: Optional[str] = None,
-    override_env_vars: Optional[Dict[str, str]] = None,
+    env: Optional[Dict[str, str]] = {},
 ) -> str:
     if apt_options is None:
         apt_options = []
@@ -362,7 +346,7 @@ def run_apt_install_command(
             + apt_options
             + packages,
             error_msg=error_msg,
-            override_env_vars=override_env_vars,
+            env=env,
         )
     except exceptions.APTProcessConflictError:
         raise exceptions.APTInstallProcessConflictError(header_msg=error_msg)
@@ -391,7 +375,7 @@ def add_auth_apt_repo(
     except ValueError:  # Then we have a bearer token
         username = "bearer"
         password = credentials
-    series = system.get_release_info().series
+    series = system.get_platform_info()["series"]
     if repo_url.endswith("/"):
         repo_url = repo_url[:-1]
     assert_valid_apt_credentials(repo_url, username, password)
@@ -417,7 +401,7 @@ def add_auth_apt_repo(
             continue  # Only enable suites matching this current series
         maybe_comment = ""
         if "-updates" in suite and not updates_enabled:
-            LOG.debug(
+            logging.debug(
                 'Not enabling apt suite "%s" because "%s-updates" is not'
                 " enabled",
                 suite,
@@ -425,8 +409,8 @@ def add_auth_apt_repo(
             )
             maybe_comment = "# "
         content += (
-            "{maybe_comment}deb {url} {suite} main\n"
-            "# deb-src {url} {suite} main\n".format(
+            "{maybe_comment}deb {url}/ubuntu {suite} main\n"
+            "# deb-src {url}/ubuntu {suite} main\n".format(
                 maybe_comment=maybe_comment, url=repo_url, suite=suite
             )
         )
@@ -603,11 +587,11 @@ def is_installed(pkg: str) -> bool:
     return pkg in get_installed_packages_names()
 
 
-def get_installed_packages() -> List[InstalledAptPackage]:
+def get_installed_packages() -> List[InstalledAptPackages]:
     out, _ = system.subp(["apt", "list", "--installed"])
     package_list = out.splitlines()[1:]
     return [
-        InstalledAptPackage(
+        InstalledAptPackages(
             name=entry.split("/")[0],
             version=entry.split(" ")[1],
             arch=entry.split(" ")[2],
@@ -757,7 +741,7 @@ def update_esm_caches(cfg) -> None:
     # Impossible to write a unittest for this because apt is globally mocked
     # in tests - down the rabbit hole, not worth it
     except apt.cache.FetchFailedException:
-        LOG.warning("Failed to fetch the ESM Apt Cache")
+        logging.warning("Failed to fetch the ESM Apt Cache")
 
 
 def remove_packages(package_names: List[str], error_message: str):
@@ -771,7 +755,7 @@ def remove_packages(package_names: List[str], error_message: str):
         ]
         + list(package_names),
         error_message,
-        override_env_vars={"DEBIAN_FRONTEND": "noninteractive"},
+        env={"DEBIAN_FRONTEND": "noninteractive"},
     )
 
 
