@@ -6,7 +6,7 @@ from typing import List
 
 from uaclient import defaults, event_logger, messages, system, util
 
-LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
+LOG = logging.getLogger(__name__)
 event = event_logger.get_event_logger()
 NoticeFileDetails = namedtuple(
     "NoticeFileDetails", ["order_id", "label", "is_permanent", "message"]
@@ -194,7 +194,7 @@ class NoticesManager:
                         notices.append(notice.value.message)
                     except Exception:
                         with util.disable_log_to_console():
-                            LOG.warning(
+                            logging.warning(
                                 "Something went wrong while processing"
                                 " notice: {}.".format(
                                     notice_file_name,
