@@ -290,7 +290,7 @@ class NetplanApply(utils.NetplanCommand):
                 utils.ip_addr_flush(iface)
             # clear NM state, especially the [device].managed=true config, as that might have been
             # re-set via an udev rule setting "NM_UNMANAGED=1"
-            shutil.rmtree('/run/NetworkManager/devices', ignore_errors=True)
+            subprocess.check_call(['rm', '-rf', '/run/NetworkManager/devices'])
             utils.systemctl_network_manager('start', sync=sync)
             if sync:
                 # 'nmcli' could be /usr/bin/nmcli or
@@ -299,7 +299,7 @@ class NetplanApply(utils.NetplanCommand):
                 # wait a bit for 'connected (site/local-only)' or
                 # 'connected' to appear in 'nmcli general' STATE
                 for _ in range(10):
-                    out = subprocess.run(cmd, capture_output=True, text=True)
+                    out = subprocess.run(cmd, capture_output=True, universal_newlines=True)
                     # Handle nmcli's "not running" return code (8) gracefully,
                     # giving some more time for NetworkManager startup
                     if out.returncode == 8:
@@ -414,4 +414,4 @@ class NetplanApply(utils.NetplanCommand):
             if exit_on_error:
                 sys.exit(1)
         except OvsDbServerNotRunning as e:
-            logging.warning('Cannot call Open vSwitch: {}.'.format(e))
+            logging.warning('Cannot call openvswitch: {}.'.format(e))
