@@ -20,7 +20,6 @@
 import ipaddress
 import json
 import logging
-import re
 import socket
 import subprocess
 import sys
@@ -28,28 +27,21 @@ from typing import Union, Dict, List, Type
 
 import dbus
 import yaml
+from rich.console import Console
+from rich.highlighter import RegexHighlighter
+from rich.theme import Theme
 
 import netplan.cli.utils as utils
 
 JSON = Union[Dict[str, 'JSON'], List['JSON'], int, str, float, bool, Type[None]]
 
 
-MATCH_TAGS = re.compile(r'\[([a-z0-9]+)\].*\[\/\1\]')
-RICH_OUTPUT = False
-try:
-    from rich.console import Console
-    from rich.highlighter import RegexHighlighter
-    from rich.theme import Theme
-
-    class NetplanHighlighter(RegexHighlighter):
-        base_style = 'netplan.'
-        highlights = [
-            r'(^|[\s\/])(?P<int>\d+)([\s:]?\s|$)',
-            r'(?P<str>(\"|\').+(\"|\'))',
-            ]
-    RICH_OUTPUT = True
-except ImportError:  # pragma: nocover (we mock RICH_OUTPUT, ignore the logging)
-    logging.debug("python3-rich not found, falling back to plain output")
+class NetplanHighlighter(RegexHighlighter):
+    base_style = 'netplan.'
+    highlights = [
+        r'(^|[\s\/])(?P<int>\d+)([\s:]?\s|$)',
+        r'(?P<str>(\"|\').+(\"|\'))',
+        ]
 
 
 class Interface():
@@ -164,7 +156,8 @@ class Interface():
     def query_networkctl(self, ifname: str) -> str:
         output: str = None
         try:
-            output = subprocess.check_output(['networkctl', 'status', '--', ifname], text=True)
+            output = subprocess.check_output(['networkctl', 'status', ifname],
+                                             universal_newlines=True)
         except Exception as e:
             logging.warning('Cannot query networkctl for {}: {}'.format(
                 ifname, str(e)))
@@ -365,7 +358,7 @@ class NetplanStatus(utils.NetplanCommand):
         data: JSON = None
         try:
             output: str = subprocess.check_output(['ip', '-d', '-j', 'addr'],
-                                                  text=True)
+                                                  universal_newlines=True)
             data = self.process_generic(output)
         except Exception as e:
             logging.critical('Cannot query iproute2 interface data: {}'.format(str(e)))
@@ -378,7 +371,7 @@ class NetplanStatus(utils.NetplanCommand):
         data: JSON = None
         try:
             output: str = subprocess.check_output(['networkctl', '--json=short'],
-                                                  text=True)
+                                                  universal_newlines=True)
             data = self.process_networkd(output)
         except Exception as e:
             logging.critical('Cannot query networkd interface data: {}'.format(str(e)))
@@ -416,10 +409,10 @@ class NetplanStatus(utils.NetplanCommand):
         data6 = None
         try:
             output4: str = subprocess.check_output(['ip', '-d', '-j', 'route'],
-                                                   text=True)
+                                                   universal_newlines=True)
             data4: JSON = self.process_generic(output4)
             output6: str = subprocess.check_output(['ip', '-d', '-j', '-6', 'route'],
-                                                   text=True)
+                                                   universal_newlines=True)
             data6: JSON = self.process_generic(output6)
         except Exception as e:
             logging.debug('Cannot query iproute2 route data: {}'.format(str(e)))
@@ -439,33 +432,20 @@ class NetplanStatus(utils.NetplanCommand):
             logging.debug('Cannot query resolved DNS data: {}'.format(str(e)))
         return (addresses, search)
 
-    def plain_print(self, *args, **kwargs):
-        if len(args):
-            lst = list(args)
-            for tag in MATCH_TAGS.findall(lst[0]):
-                # remove matching opening and closing tag
-                lst[0] = lst[0].replace('[{}]'.format(tag), '')\
-                               .replace('[/{}]'.format(tag), '')
-            return print(*lst, **kwargs)
-        return print(*args, **kwargs)
-
     def pretty_print(self, data: JSON, total: int, _console_width=None) -> None:
-        if RICH_OUTPUT:
-            # TODO: Use a proper (subiquity?) color palette
-            theme = Theme({
-                'netplan.int': 'bold cyan',
-                'netplan.str': 'yellow',
-                'muted': 'grey62',
-                'online': 'green bold',
-                'offline': 'red bold',
-                'unknown': 'yellow bold',
-                'highlight': 'bold'
-                })
-            console = Console(highlighter=NetplanHighlighter(), theme=theme,
-                              width=_console_width, emoji=False)
-            pprint = console.print
-        else:
-            pprint = self.plain_print
+        # TODO: Use a proper (subiquity?) color palette
+        theme = Theme({
+            'netplan.int': 'bold cyan',
+            'netplan.str': 'yellow',
+            'muted': 'grey62',
+            'online': 'green bold',
+            'offline': 'red bold',
+            'unknown': 'yellow bold',
+            'highlight': 'bold'
+            })
+        console = Console(highlighter=NetplanHighlighter(), theme=theme,
+                          width=_console_width, emoji=False)
+        pprint = console.print
 
         pad = '18'
         global_state = data.get('netplan-global-state', {})
