@@ -45,12 +45,15 @@ def is_nm_snap_enabled():
 
 
 def nmcli(args):  # pragma: nocover (covered in autopkgtest)
-    binary_name = 'nmcli'
+    # 'nmcli' could be /usr/bin/nmcli or /snap/bin/nmcli -> /snap/bin/network-manager.nmcli
+    # PATH is defined in cli/core.py
+    subprocess.check_call(['nmcli'] + args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
-    if is_nm_snap_enabled():
-        binary_name = 'network-manager.nmcli'
 
-    subprocess.check_call([binary_name] + args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+def nmcli_out(args: list) -> str:  # pragma: nocover (covered in autopkgtest)
+    # 'nmcli' could be /usr/bin/nmcli or /snap/bin/nmcli -> /snap/bin/network-manager.nmcli
+    # PATH is defined in cli/core.py
+    return subprocess.check_output(['nmcli'] + args, universal_newlines=True)
 
 
 def nm_running():  # pragma: nocover (covered in autopkgtest)
@@ -85,7 +88,7 @@ def systemctl_network_manager(action, sync=False):
     return systemctl(action, [NM_SERVICE_NAME], sync)  # pragma: nocover (covered in autopkgtest)
 
 
-def systemctl(action, services, sync=False):
+def systemctl(action: str, services: list, sync: bool = False):
     if len(services) >= 1:
         command = ['systemctl', action]
 
@@ -123,6 +126,16 @@ def systemctl_is_active(unit_pattern):
     return False
 
 
+def systemctl_is_masked(unit_pattern):
+    '''Return True if output is "masked" or "masked-runtime"'''
+    res = subprocess.run(['systemctl', 'is-enabled', unit_pattern],
+                         stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                         universal_newlines=True)
+    if res.returncode > 0 and 'masked' in res.stdout:
+        return True
+    return False
+
+
 def systemctl_daemon_reload():
     '''Reload systemd unit files from disk and re-calculate its dependencies'''
     subprocess.check_call(['systemctl', 'daemon-reload'])
